/*
	Pinguino + MOD RTC
	Example
	
	Read/Set the Date/Time
	with the MOD-RTC module
	and its EEPROM chip
	
	See the Readme file for
	more information.
	
	If you have any questions, email
	support@olimex.com
	
	OLIMEX, June 2012
	http://olimex.com/dev	
*/

#include <serial.c>
#include <__cdc.c>

#if ( defined(PIC32_PINGUINO_OTG) || defined(PIC32_PINGUINO) || defined(PIC32_PINGUINO_MICRO) )

	//Definitions for 
	// -- PIC32-PINGUINO
	// -- PIC32-PINGUINO-OTG
	// -- PIC32-PINGUINO-MICRO
	#define BUTTON1INIT TRISDbits.TRISD0 = 1
	
	//YELLOW LED is D1
	#define YLEDINIT   TRISDCLR = 0x02;
	#define YLED1      PORTDSET = 0x02;
	#define YLED0      PORTDCLR = 0x02;
	#define YLEDSWITCH PORTD   ^= 0x02;
	
	//GREEN LED is B15
	#define GLEDINIT  	TRISGCLR = 0x40;
	#define GLED1 		PORTGSET = 0x40;
	#define GLED0 		PORTGCLR = 0x40;
	#define GLEDSWITCH  PORTG   ^= 0x40;
	
	#define YELLOW 0
	#define GREEN  1
	#define BOTH   2
#endif

/*  Prototypes and useful functions     */
		
	//Sets the date and time to the specified values
	//if the value is outside the allowed range 
	//it will be set to zero
	//returns 1 if successful, 0 if there was an ERROR while writing
	// ranges: sec(0-59), min (0-59), hr(0-23), day(0-31), weekday(0-6,Sun=0)
	//		   month(1-12), year(0-99)
	
	boolean set_time_date(unsigned char sec, unsigned char mint,
						  unsigned char hr , unsigned char day,
						  unsigned char wd , unsigned char month,
						  unsigned char year );
	

	// Gets the current time (in decimal format)
	// returns 1 if successful, 0 if there was an ERROR while reading
	boolean get_time_date( unsigned char *sec, unsigned char *mint,
						   unsigned char *hr , unsigned char *day,
						   unsigned char *wd , unsigned char *month,
						   unsigned char *year );
						   
	// Prints the current time on the terminal
	void print_time();
	
	// Prints the current date and time
	// Doesn't return anything; Will print an ERROR
	// message if the function that reads the time 
	// returned an ERROR
	
	
	//A macro that waits for a key via CDC
	//Works with all pinguinos that support CDC
	//Syntax:
	//   WAIT;   //-> pauses the program and waits
				 //   until a key is pressed 

	//Converts a number to the BCD format tttt|uuuu (tens and units)
	unsigned char char_to_bcd(unsigned char a);
	
	//WAIT Macro 
	#define WAIT wait_123()
	char ch123;
	
	void wait_123() __attribute ((noinline));
	
	void wait_123(){ ch123 = CDCgetkey(); }
	//END Wait Macro
	
	//Blinks an LED five times - mode is YELLOW, GREEN or BOTH
	void blinkled(unsigned char mode);
	
	const char * wDays[7] = {"Sunday", "Monday", "Tuesday", "Wednesday", 
							 "Thursday", "Friday", "Saturday"};
	const char * format_line = "----------------------------------------";
	
	/* I2C EEPROM */
#if ( defined(PIC32_PINGUINO_OTG) || defined(PIC32_PINGUINO) )
	#define SDA_TRIS TRISBbits.TRISB8
	#define SDA_LAT LATBbits.LATB8
	#define SDA_PORT PORTBbits.RB8

	#define SCL_TRIS TRISBbits.TRISB9
	#define SCL_LAT LATBbits.LATB9
	#define SCL_PORT PORTBbits.RB9

	#define SLAVE_WRITE 0xA2
	#define SLAVE_READ  0xA3

#else if( defined(PIC32_PINGUINO_MICRO))
	
	#define SDA_TRIS TRISDbits.TRISD9
	#define SDA_LAT LATDbits.LATD9
	#define SDA_PORT PORTDbits.RD9

	#define SCL_TRIS TRISDbits.TRISD10
	#define SCL_LAT LATDbits.LATD10
	#define SCL_PORT PORTDbits.RD10

	#define SLAVE_WRITE 0xA2
	#define SLAVE_READ  0xA3
	
#endif

	#define ACK 0
	#define NACK 1

	#define UNIT 0x0f

	static void local_I2C_Delay(void);
	static void local_I2C_Start(void);
	static void local_I2C_Stop(void);
	static byte local_I2C_ReadByte(char ack);
	static char local_I2C_WriteByte(byte data); // returns ack state
	static void wait40ms(void);
	static void MakeCalculations(void);

	/* End I2C EEPROM */

	void message(const char * m)
	{
		delay(10);
		CDC.println(m);
		delay(10);
	}

/* Main Body */	
	
void setup()
{	
	/* Init the Module, LEDs and Button on the pinguino board*/
	YLEDINIT;
	GLEDINIT;	
	YLED0;
	GLED0;
	BUTTON1INIT;
		
	WAIT;
	message("Press a key to set the time...");
	
	boolean flag=1;
	//Setting the time and date
	flag = set_time_date(0,0,0,0,1,1,0);
	
	
}

void loop()
{	
	message("Press a key to print the time: ");
	WAIT;
	
	print_time();
	
}

/* Function Definitions */

void blinkled(unsigned char mode)
{		
	//Turn off all LEDs
	YLED0;
	GLED0;
	
	char i;
	
	// Blink
	switch(mode)
	{
		case YELLOW:
			for(i=0; i<10;i++)
			{
				YLEDSWITCH;
				delay(100);
			}
			break;
		
		case GREEN:
			for(i=0; i<10;i++)
			{
				GLEDSWITCH;
				delay(100);
			}
			break;
		
		case BOTH:
			for(i=0; i<10;i++)
			{
				YLEDSWITCH;
				GLEDSWITCH;
				delay(100);
			}
			break;
	}			
				
}

boolean set_time_date(unsigned char sec, unsigned char mint,
					  unsigned char hr , unsigned char day,
					  unsigned char wd , unsigned char month,
					  unsigned char year )
{
	if(sec>59) sec = 0;
	if(mint>59) mint = 0;
	if(hr>23 ) hr = 0;
	if(wd>6)   wd = 0;
	if(month>12 || month == 0) month = 1;
	if(year>99) year = 0;
	
	sec = char_to_bcd(sec); mint = char_to_bcd(mint);
	hr = char_to_bcd(hr); day = char_to_bcd(day);
	wd = char_to_bcd(wd); month = char_to_bcd(month);
	year = char_to_bcd(year);
	
	SDA_TRIS = 1;
	SCL_TRIS = 1;
	
	SDA_LAT = 0;
	SCL_LAT = 0;

	boolean aflag; // the ACK flag
	
	delay(20);
	
	local_I2C_Start();
	aflag = local_I2C_WriteByte(SLAVE_WRITE); 
	aflag|= local_I2C_WriteByte(0x02);			
	aflag|= local_I2C_WriteByte(sec);  
	aflag|= local_I2C_WriteByte(mint);
	aflag|= local_I2C_WriteByte(hr);
	aflag|= local_I2C_WriteByte(day);
	aflag|= local_I2C_WriteByte(wd);
	aflag|= local_I2C_WriteByte(month);
	aflag|= local_I2C_WriteByte(year);
	local_I2C_Stop();
	
	delay(1000);
	
	return !aflag;
}

boolean get_time_date( unsigned char *sec, unsigned char *mint,
						unsigned char *hr , unsigned char *day,
						unsigned char *wd , unsigned char *month,
						unsigned char *year )
{
	//acknowledge flag
	boolean aflag;
	
	SDA_TRIS = 1;
	SCL_TRIS = 1;
	
	SDA_LAT = 0;
	SCL_LAT = 0;
		
	delay(20);
	
	// Read operation
	local_I2C_Start();
	aflag = local_I2C_WriteByte(SLAVE_WRITE);
	aflag |= local_I2C_WriteByte(0x02);
	local_I2C_Stop();
	local_I2C_Start();
	aflag |= local_I2C_WriteByte(SLAVE_READ);
	*sec = local_I2C_ReadByte(ACK);
    *mint = local_I2C_ReadByte(ACK);	
	*hr  = local_I2C_ReadByte(ACK);
	*day = local_I2C_ReadByte(ACK);
	*wd = local_I2C_ReadByte(ACK);
	*month = local_I2C_ReadByte(ACK);
	*year = local_I2C_ReadByte(NACK);
	local_I2C_Stop();
	
	// Converting raw BCD data to decimal values
	*sec   =  ((  (*sec)&0x70)>>4)*10    +   ((*sec)&UNIT);
	*mint  =  (( (*mint)&0x70)>>4)*10    +  ((*mint)&UNIT);
	*hr    =  ((   (*hr)&0x30)>>4)*10    +   ( (*hr)&UNIT);
	*day   =  ((  (*day)&0x30)>>4)*10    +   ((*day)&UNIT);
	*wd    =       (*wd)&0x07;
	*month =  (((*month)&0x10)>>4)*10    + ((*month)&UNIT);
	*year  =  (( (*year)&0xF0)>>4)*10    +  ((*year)&UNIT);
	
	
	return !aflag;
}

void print_time()
{
	unsigned char sec, mint, hr, day, wd, month, year;
	
	boolean flag = 1;
	flag = get_time_date(&sec, &mint, &hr, &day, &wd, &month, &year);

	
	if(!flag)
	{
		message(format_line);
		message("ERROR!!!");
		message(format_line);

		
	}else
		{
			message("");
			message(format_line);
			message("Time & Date: ");
			CDC.printf("%d:%d:%d \r\n", hr, mint, sec);
			CDC.printf("D/M/Y: %d.%d.%d \r\n ", day, month, year);
			CDC.printf("Day of the week: "); CDC.print(wDays[wd]);
			message("");
			message(format_line);
					
	    }
}

unsigned char char_to_bcd(unsigned char a)
{	
	return ((a/10)<<4) | (a%10);	
}


/* I2C EEPROM CODE */

static void wait40ms(void)
{
	delayms(40);
	
	/*UINT d = 40000;
	while(d--) {
		Nop();
	}*/		
}	

static void local_I2C_Delay(void)
{   
    delayMicroseconds(500);
	/*UINT d = 100;
	while(d--) {
		Nop();
	}*/	
}	

static void local_I2C_Start(void)
{
	SDA_TRIS = 0;
	local_I2C_Delay();
	SCL_TRIS = 0;
	local_I2C_Delay();
	
}

static void local_I2C_Stop(void)
{
	SDA_TRIS = 0;
	local_I2C_Delay();
	SCL_TRIS = 1;
	local_I2C_Delay();
	SDA_TRIS = 1;
	local_I2C_Delay();
}

static byte local_I2C_ReadByte(char ack)
{
	byte data = 0;
	char i;
	
	SDA_TRIS = 1;
	for(i = 0; i < 8; i++) {
		local_I2C_Delay();
		SCL_TRIS = 1;
		local_I2C_Delay();
		data |= SDA_PORT & 0x01;
		if(i != 7)
			data <<= 1;
		SCL_TRIS = 0;
	}
	
	// read the ack
	local_I2C_Delay();
	SDA_TRIS = ack;
	local_I2C_Delay();
	SCL_TRIS = 1;
	local_I2C_Delay();
	SCL_TRIS = 0;
	local_I2C_Delay();
	
	return data;
}

// returns ack state, 0 means acknowledged
static char local_I2C_WriteByte(byte data)
{
	char i;

	// send the 8 bits
	for(i = 0; i < 8; i++) {
		SDA_TRIS = (data & 0x80) ? 1 : 0;
		data <<= 1;
		local_I2C_Delay();
		SCL_TRIS = 1;
		local_I2C_Delay();
		SCL_TRIS = 0;
	}
	
	// read the ack
	SDA_TRIS = 1;
	local_I2C_Delay();
	SCL_TRIS = 1;
	local_I2C_Delay();
	i = SDA_PORT;
	SCL_TRIS = 0;
	local_I2C_Delay();
	
	return i;
}

/* END I2C EEPROM CODE */